@@ -633,7 +633,7 @@ ngx_conf_include(ngx_conf_t *cf, ngx_command_t *cmd, void *conf)
 {
     char        *rv;
     ngx_int_t    n;
-    ngx_str_t   *value, file;
+    ngx_str_t   *value, file, name;
     ngx_glob_t   gl;
 
     value = cf->args->elts;
@@ -641,14 +641,22 @@ ngx_conf_include(ngx_conf_t *cf, ngx_command_t *cmd, void *conf)
 
     ngx_log_debug1(NGX_LOG_DEBUG_CORE, cf->log, 0, "include %s", file.data);
 
-    if (ngx_conf_full_name(cf->cycle, &file) == NGX_ERROR) {
+    if (ngx_conf_full_name(cf->cycle, &file, 1) == NGX_ERROR) {
         return NGX_CONF_ERROR;
     }
 
+    if (strpbrk((char *) file.data, "*?[") == NULL) {
+
+        ngx_log_debug1(NGX_LOG_DEBUG_CORE, cf->log, 0, "include %s", file.data);
+
+        return ngx_conf_parse(cf, &file);
+    }
+
     ngx_memzero(&gl, sizeof(ngx_glob_t));
 
     gl.pattern = file.data;
     gl.log = cf->log;
+    gl.test = 1;
 
     if (ngx_open_glob(&gl) != NGX_OK) {
         ngx_conf_log_error(NGX_LOG_EMERG, cf, ngx_errno,
@@ -659,12 +667,15 @@ ngx_conf_include(ngx_conf_t *cf, ngx_command_t *cmd, void *conf)
     rv = NGX_CONF_OK;
 
     for ( ;; ) {
-        n = ngx_read_glob(&gl, &file);
+        n = ngx_read_glob(&gl, &name);
 
         if (n != NGX_OK) {
             break;
         }
 
+        file.len = name.len++;
+        file.data = ngx_pstrdup(cf->pool, &name);
+
         ngx_log_debug1(NGX_LOG_DEBUG_CORE, cf->log, 0, "include %s", file.data);
 
         rv = ngx_conf_parse(cf, &file);
@@ -681,9 +692,10 @@ ngx_conf_include(ngx_conf_t *cf, ngx_command_t *cmd, void *conf)
 
 
 ngx_int_t
-ngx_conf_full_name(ngx_cycle_t *cycle, ngx_str_t *name)
+ngx_conf_full_name(ngx_cycle_t *cycle, ngx_str_t *name, ngx_uint_t conf_prefix)
 {
-    u_char     *p;
+    size_t      len;
+    u_char     *p, *prefix;
     ngx_str_t   old;
 
     if (name->data[0] == '/') {
@@ -704,14 +716,22 @@ ngx_conf_full_name(ngx_cycle_t *cycle, ngx_str_t *name)
 
     old = *name;
 
-    name->len = cycle->root.len + old.len;
+    if (conf_prefix) {
+        len = sizeof(NGX_CONF_PREFIX) - 1;
+        prefix = (u_char *) NGX_CONF_PREFIX;
 
+    } else {
+        len = cycle->root.len;
+        prefix = cycle->root.data;
+    }
+
+    name->len = len + old.len;
     name->data = ngx_palloc(cycle->pool, name->len + 1);
     if (name->data == NULL) {
         return  NGX_ERROR;
     }
 
-    p = ngx_cpymem(name->data, cycle->root.data, cycle->root.len),
+    p = ngx_cpymem(name->data, prefix, len);
     ngx_cpystrn(p, old.data, old.len + 1);
 
     return NGX_OK;
@@ -734,7 +754,7 @@ ngx_conf_open_file(ngx_cycle_t *cycle, ngx_str_t *name)
     if (name) {
         full = *name;
 
-        if (ngx_conf_full_name(cycle, &full) == NGX_ERROR) {
+        if (ngx_conf_full_name(cycle, &full, 0) == NGX_ERROR) {
             return NULL;
         }
 
@@ -925,7 +945,7 @@ ngx_conf_set_str_array_slot(ngx_conf_t *cf, ngx_command_t *cmd, void *conf)
 
     a = (ngx_array_t **) (p + cmd->offset);
 
-    if (*a == NULL) {
+    if (*a == NGX_CONF_UNSET_PTR) {
         *a = ngx_array_create(cf->pool, 4, sizeof(ngx_str_t));
         if (*a == NULL) {
             return NGX_CONF_ERROR;
