@@ -27,16 +27,19 @@ struct dvpoll {
 
 
 typedef struct {
-    u_int  changes;
-    u_int  events;
+    ngx_uint_t      changes;
+    ngx_uint_t      events;
 } ngx_devpoll_conf_t;
 
 
 static ngx_int_t ngx_devpoll_init(ngx_cycle_t *cycle, ngx_msec_t timer);
 static void ngx_devpoll_done(ngx_cycle_t *cycle);
-static ngx_int_t ngx_devpoll_add_event(ngx_event_t *ev, int event, u_int flags);
-static ngx_int_t ngx_devpoll_del_event(ngx_event_t *ev, int event, u_int flags);
-static ngx_int_t ngx_devpoll_set_event(ngx_event_t *ev, int event, u_int flags);
+static ngx_int_t ngx_devpoll_add_event(ngx_event_t *ev, ngx_int_t event,
+    ngx_uint_t flags);
+static ngx_int_t ngx_devpoll_del_event(ngx_event_t *ev, ngx_int_t event,
+    ngx_uint_t flags);
+static ngx_int_t ngx_devpoll_set_event(ngx_event_t *ev, ngx_int_t event,
+    ngx_uint_t flags);
 static ngx_int_t ngx_devpoll_process_events(ngx_cycle_t *cycle,
     ngx_msec_t timer, ngx_uint_t flags);
 
@@ -45,7 +48,7 @@ static char *ngx_devpoll_init_conf(ngx_cycle_t *cycle, void *conf);
 
 static int              dp = -1;
 static struct pollfd   *change_list, *event_list;
-static u_int            nchanges, max_changes, nevents;
+static ngx_uint_t       nchanges, max_changes, nevents;
 
 static ngx_event_t    **change_index;
 
@@ -209,7 +212,7 @@ ngx_devpoll_done(ngx_cycle_t *cycle)
 
 
 static ngx_int_t
-ngx_devpoll_add_event(ngx_event_t *ev, int event, u_int flags)
+ngx_devpoll_add_event(ngx_event_t *ev, ngx_int_t event, ngx_uint_t flags)
 {
 #if (NGX_DEBUG)
     ngx_connection_t *c;
@@ -222,7 +225,7 @@ ngx_devpoll_add_event(ngx_event_t *ev, int event, u_int flags)
 #if (NGX_DEBUG)
     c = ev->data;
     ngx_log_debug2(NGX_LOG_DEBUG_EVENT, ev->log, 0,
-                   "devpoll add event: fd:%d ev:%04Xd", c->fd, event);
+                   "devpoll add event: fd:%d ev:%04Xi", c->fd, event);
 #endif
 
     ev->active = 1;
@@ -232,7 +235,7 @@ ngx_devpoll_add_event(ngx_event_t *ev, int event, u_int flags)
 
 
 static ngx_int_t
-ngx_devpoll_del_event(ngx_event_t *ev, int event, u_int flags)
+ngx_devpoll_del_event(ngx_event_t *ev, ngx_int_t event, ngx_uint_t flags)
 {
     ngx_event_t       *e;
     ngx_connection_t  *c;
@@ -244,7 +247,7 @@ ngx_devpoll_del_event(ngx_event_t *ev, int event, u_int flags)
 #endif
 
     ngx_log_debug2(NGX_LOG_DEBUG_EVENT, ev->log, 0,
-                   "devpoll del event: fd:%d ev:%04Xd", c->fd, event);
+                   "devpoll del event: fd:%d ev:%04Xi", c->fd, event);
 
     if (ngx_devpoll_set_event(ev, POLLREMOVE, flags) == NGX_ERROR) {
         return NGX_ERROR;
@@ -282,7 +285,7 @@ ngx_devpoll_del_event(ngx_event_t *ev, int event, u_int flags)
 
 
 static ngx_int_t
-ngx_devpoll_set_event(ngx_event_t *ev, int event, u_int flags)
+ngx_devpoll_set_event(ngx_event_t *ev, ngx_int_t event, ngx_uint_t flags)
 {
     size_t             n;
     ngx_connection_t  *c;
@@ -290,7 +293,7 @@ ngx_devpoll_set_event(ngx_event_t *ev, int event, u_int flags)
     c = ev->data;
 
     ngx_log_debug3(NGX_LOG_DEBUG_EVENT, ev->log, 0,
-                   "devpoll fd:%d ev:%04Xd fl:%04Xd", c->fd, event, flags);
+                   "devpoll fd:%d ev:%04Xi fl:%04Xi", c->fd, event, flags);
 
     if (nchanges >= max_changes) {
         ngx_log_error(NGX_LOG_WARN, ev->log, 0,
@@ -307,7 +310,7 @@ ngx_devpoll_set_event(ngx_event_t *ev, int event, u_int flags)
     }
 
     change_list[nchanges].fd = c->fd;
-    change_list[nchanges].events = event;
+    change_list[nchanges].events = (short) event;
     change_list[nchanges].revents = 0;
 
     change_index[nchanges] = ev;
@@ -362,7 +365,7 @@ ngx_devpoll_process_events(ngx_cycle_t *cycle, ngx_msec_t timer,
     }
 
     dvp.dp_fds = event_list;
-    dvp.dp_nfds = nevents;
+    dvp.dp_nfds = (int) nevents;
     dvp.dp_timeout = timer;
     events = ioctl(dp, DP_POLL, &dvp);
 
