@@ -66,16 +66,19 @@ int epoll_wait(int epfd, struct epoll_event *events, int nevents, int timeout)
 
 
 typedef struct {
-    u_int  events;
+    ngx_uint_t  events;
 } ngx_epoll_conf_t;
 
 
 static ngx_int_t ngx_epoll_init(ngx_cycle_t *cycle, ngx_msec_t timer);
 static void ngx_epoll_done(ngx_cycle_t *cycle);
-static ngx_int_t ngx_epoll_add_event(ngx_event_t *ev, int event, u_int flags);
-static ngx_int_t ngx_epoll_del_event(ngx_event_t *ev, int event, u_int flags);
+static ngx_int_t ngx_epoll_add_event(ngx_event_t *ev, ngx_int_t event,
+    ngx_uint_t flags);
+static ngx_int_t ngx_epoll_del_event(ngx_event_t *ev, ngx_int_t event,
+    ngx_uint_t flags);
 static ngx_int_t ngx_epoll_add_connection(ngx_connection_t *c);
-static ngx_int_t ngx_epoll_del_connection(ngx_connection_t *c, u_int flags);
+static ngx_int_t ngx_epoll_del_connection(ngx_connection_t *c,
+    ngx_uint_t flags);
 static ngx_int_t ngx_epoll_process_events(ngx_cycle_t *cycle, ngx_msec_t timer,
     ngx_uint_t flags);
 
@@ -84,7 +87,7 @@ static char *ngx_epoll_init_conf(ngx_cycle_t *cycle, void *conf);
 
 static int                  ep = -1;
 static struct epoll_event  *event_list;
-static u_int                nevents;
+static ngx_uint_t           nevents;
 
 
 static ngx_str_t      epoll_name = ngx_string("epoll");
@@ -205,7 +208,7 @@ ngx_epoll_done(ngx_cycle_t *cycle)
 
 
 static ngx_int_t
-ngx_epoll_add_event(ngx_event_t *ev, int event, u_int flags)
+ngx_epoll_add_event(ngx_event_t *ev, ngx_int_t event, ngx_uint_t flags)
 {
     int                  op;
     uint32_t             events, prev;
@@ -240,7 +243,7 @@ ngx_epoll_add_event(ngx_event_t *ev, int event, u_int flags)
         op = EPOLL_CTL_ADD;
     }
 
-    ee.events = events | flags;
+    ee.events = events | (uint32_t) flags;
     ee.data.ptr = (void *) ((uintptr_t) c | ev->instance);
 
     ngx_log_debug3(NGX_LOG_DEBUG_EVENT, ev->log, 0,
@@ -263,7 +266,7 @@ ngx_epoll_add_event(ngx_event_t *ev, int event, u_int flags)
 
 
 static ngx_int_t
-ngx_epoll_del_event(ngx_event_t *ev, int event, u_int flags)
+ngx_epoll_del_event(ngx_event_t *ev, ngx_int_t event, ngx_uint_t flags)
 {
     int                  op;
     uint32_t             prev;
@@ -295,7 +298,7 @@ ngx_epoll_del_event(ngx_event_t *ev, int event, u_int flags)
 
     if (e->active) {
         op = EPOLL_CTL_MOD;
-        ee.events = prev | flags;
+        ee.events = prev | (uint32_t) flags;
         ee.data.ptr = (void *) ((uintptr_t) c | ev->instance);
 
     } else {
@@ -345,10 +348,10 @@ ngx_epoll_add_connection(ngx_connection_t *c)
 
 
 static ngx_int_t
-ngx_epoll_del_connection(ngx_connection_t *c, u_int flags)
+ngx_epoll_del_connection(ngx_connection_t *c, ngx_uint_t flags)
 {
-    int                  op;
-    struct epoll_event   ee;
+    int                 op;
+    struct epoll_event  ee;
 
     /*
      * when the file descriptor is closed the epoll automatically deletes
@@ -399,7 +402,7 @@ ngx_epoll_process_events(ngx_cycle_t *cycle, ngx_msec_t timer, ngx_uint_t flags)
     ngx_log_debug1(NGX_LOG_DEBUG_EVENT, cycle->log, 0,
                    "epoll timer: %M", timer);
 
-    events = epoll_wait(ep, event_list, nevents, timer);
+    events = epoll_wait(ep, event_list, (int) nevents, timer);
 
     if (events == -1) {
         err = ngx_errno;
