@@ -718,7 +718,7 @@ ngx_mail_auth_http_process_headers(ngx_mail_session_t *s,
                     return;
                 }
 
-                ngx_add_timer(s->connection->read, timer * 1000);
+                ngx_add_timer(s->connection->read, (ngx_msec_t) (timer * 1000));
 
                 s->connection->read->handler = ngx_mail_auth_sleep_handler;
 
@@ -735,7 +735,7 @@ ngx_mail_auth_http_process_headers(ngx_mail_session_t *s,
                     return;
                 }
 
-                ngx_add_timer(s->connection->read, timer * 1000);
+                ngx_add_timer(s->connection->read, (ngx_msec_t) (timer * 1000));
 
                 s->connection->read->handler = ngx_mail_auth_sleep_handler;
 
@@ -1368,7 +1368,7 @@ ngx_mail_auth_http(ngx_conf_t *cf, ngx_command_t *cmd, void *conf)
         u.url.data += 7;
     }
 
-    if (ngx_parse_url(cf, &u) != NGX_OK) {
+    if (ngx_parse_url(cf->pool, &u) != NGX_OK) {
         if (u.err) {
             ngx_conf_log_error(NGX_LOG_EMERG, cf, 0,
                                "%s in auth_http \"%V\"", u.err, &u.url);
